Package: modelDown
Title: Make Static HTML Website for Predictive Models
Version: 1.1
Authors@R: c(person("Przemysław", "Biecek", email = "przemyslaw.biecek@gmail.com", role = c("aut")), 
    person("Magda", "Tatarynowicz", email = "magda.tatarynowicz@gmail.com", role = c("aut")), 
    person("Kamil", "Romaszko", email = "kamil.romaszko@gmail.com", role = c("aut", "cre")), 
    person("Mateusz", "Urbański", email = "matiszak@gmail.com", role = c("aut")))
Description: Website generator with HTML summaries for predictive models.
    This package uses 'DALEX' explainers to describe global model behavior. 
    We can see how well models behave (tabs: Model Performance, Auditor),
    how much each variable contributes to predictions (tabs: Variable Response) 
    and which variables are the most important for a given model (tabs: Variable Importance).
    We can also compare Concept Drift for pairs of models (tabs: Drifter).
    Additionally, data available on the website can be easily recreated in current R session.
    Work on this package was financially supported by the NCN Opus grant 2017/27/B/ST6/01307 
    at Warsaw University of Technology, Faculty of Mathematics and Information Science.
Depends: R (>= 3.4.0)
License: Apache License 2.0
Encoding: UTF-8
LazyData: true
Imports: DALEX (>= 1.0), auditor (>= 0.3.0), ggplot2 (>= 3.1.0),
        whisker (>= 0.3-2), DT (>= 0.4), kableExtra (>= 0.9.0), psych
        (>= 1.8.4), archivist (>= 2.1.0), svglite (>= 1.2.1), devtools
        (>= 2.0.1), breakDown (>= 0.1.6), drifter (>= 0.2.1)
Suggests: ranger, testthat, useful
RoxygenNote: 7.1.0
URL: https://github.com/ModelOriented/modelDown
BugReports: https://github.com/ModelOriented/modelDown/issues
NeedsCompilation: no
Packaged: 2020-04-11 16:08:24 UTC; kromash
Author: Przemysław Biecek [aut],
  Magda Tatarynowicz [aut],
  Kamil Romaszko [aut, cre],
  Mateusz Urbański [aut]
Maintainer: Kamil Romaszko <kamil.romaszko@gmail.com>
Repository: CRAN
Date/Publication: 2020-04-15 00:30:03 UTC
Built: R 4.6.0; ; 2026-04-29 22:07:42 UTC; unix
